/*************************************************************** 
* 
* CompoundName.h             & XMM Science Analysis System (SAS) 
* 
* Various CompoundName utilities
* 
* (c) 2000 ESA - Written by Mark Thomas, CARA
* 
* $Id: CompoundName.h,v 1.2 2004/03/05 14:43:00 eojero Exp $ 
* 
*****************************************************************/ 
#ifndef COMPOUND_NAME_H
#define COMPOUND_NAME_H

#include <SimpleScanner.h>

#include "Dal.h"

class CompoundName
{
	public :

		enum SyntaxType {
			Any,
			AnyAttributableName,
			AnyAttributeName,
			AnyDataComponentName,
			ArrayName,
			//ArrayAttributeName,
			//BadSyntax,
			BlockAttributeName,
			BlockName,
			ColumnAttributeName,
			ColumnName,
			DataSetAttributeName,
			DataSetName,
			TableName
			//TableAttributeName
		};

		CompoundName(
			const std::string&			name,
			SyntaxType			expected = Any,
			DataSet*			dataSet = 0,
			Dal::AccessMode 		accessMode = Dal::Read,
			DataSetServer::MemoryModel	memoryModel = DataSetServer::UseEnvironment );

		~CompoundName();

		SyntaxType state() const;

		Attributable* attributable() const;
		Column* column() const;
		Array* array() const;
		Table* table() const;
		Block* block() const;
		DataSet* dataSet() const;
		Attribute* attribute() const;

		std::string dataSetName() const;
		std::string blockName() const;
		std::string columnName() const;
		std::string attributeName() const;

		std::string qualifiedBlockName() const;
		std::string qualifiedColumnName() const;
		std::string qualifiedAttributeName() const;
		std::string qualifiedAttributableName() const;

		//bool exists() const;

	private :

		std::string				_name;
		SyntaxType			_expected;
		Dal::AccessMode			_accessMode;
		DataSetServer::MemoryModel	_memoryModel;
		DataSet*			_dataSet;
		mutable DataSet*		_set;
		//Block*		_block;
		//Array*		_array;
		//Table*		_table;
		//Column*		_column;
		//Attribute*	_attribute;
		//Attributable*	_attributable;
		SyntaxType			_state;
		std::string				_dataSetName;
		std::string				_blockName;
		std::string				_columnName;
		std::string				_attributeName;

		void parseCompoundName();
		void parseColumnName( SimpleScanner& sc );
		void parseBlockName( SimpleScanner& sc );
		void parseDataSetName( SimpleScanner& sc );
		void parseAttributeName( SimpleScanner& sc );

		// Disallow
		CompoundName( const CompoundName& );
		CompoundName& operator=( const CompoundName&);
};

#endif

