/*************************************************************** 
* 
* DalColumn.h               & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Column.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalColumn.h,v 1.2 2004/03/05 14:43:01 eojero Exp $
* 
*****************************************************************/ 
#ifndef DAL_COLUMN_H
#define DAL_COLUMN_H


#include <Dal.h>

#include "DalDataComponent.h"
#include "DalAttributable.h"
#include "DalTable.h"


//class DalColumn : public virtual Column, public DalDataComponent, public DalAttributable, public DalChild<Table>, public DalChild<DalTable>
class DalColumn : public virtual Column, public DalAttributable, public DalDataComponent, public DalChild<Table>, public DalChild<DalTable>
{
	public :
		DalColumn(
				const std::string &			name,
				DataType			dataType,
				const longVec&			size,
				const std::string &			label,
				const std::string &			units,
				const DalTable *		parent );
		DalColumn(
				const DalColumn&		column,
				DalTable *			parent,
				std::string				newName = "" );
		virtual ~DalColumn(){}
		virtual DalColumn& operator=( const Column& );

		ColumnData * data(
				unsigned long			from=0,
				unsigned long			count=0,
				AccessMode			accessMode=AsParent ) const;

    		ColumnData * rowData(
    			unsigned long				rowNumber,
    			AccessMode				accessMode=AsParent ) const;

		CellData * cellData(
				unsigned long			rowNumber,
				unsigned long			from=0,
				unsigned long			count=0,
				AccessMode			accessMode=AsParent ) const;

		std::string parseName( const std::string & compoundName ) const;
		std::string qualifiedName() const;
		unsigned int columnNumber() const;
		void columnNumber( unsigned int number );
		CellType cellType() const;

		virtual void insertRows(
				unsigned long			pos,
				unsigned long 			count=1 ) = 0;

		virtual void deleteRows(
				unsigned long 			pos,
				unsigned long 			count=1 ) = 0;

		virtual void copyRows(
				unsigned long 			from,
				unsigned long 			to,
				unsigned long 			count=1 ) = 0;

		//virtual void releaseAll() = 0;
		virtual void printOn( ostream& os ) const = 0;

	private :

		CellType	_cellType;
		unsigned int	_columnNumber;

		virtual CellData * makeCellData(
				unsigned long			rowNumber,
				unsigned long			from,
				unsigned long			count,
				AccessMode			accessMode ) const = 0;

		virtual ColumnData * makeColumnData(
				unsigned long			from,
				unsigned long			count,
				AccessMode			accessMode ) const = 0;


};


#endif


