/*************************************************************** 
* 
* HiLowArrayData.h             & XMM Science Analysis System (SAS) 
* 
* Class representing array data object.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowArrayData.h,v 1.2 2004/03/05 14:43:02 eojero Exp $
* 
*****************************************************************/ 
#ifndef HILOW_ARRAY_DATA_H
#define HILOW_ARRAY_DATA_H


#include <Dal.h>

#include "HiLowData.h"
#include "HiLowDataStorage.h"
#include "DalArrayData.h"
#include "DalArray.h"


class HiLowArrayData : public DalArrayData, public HiLowData<DalArray>
{
	public :

		HiLowArrayData(
			HiLowDataStorage *		data,
			const DalArray *		parent,
			const longVec&			from,
			const longVec&			count,
			AccessMode			accessMode );

		virtual ~HiLowArrayData();

		int8 * int8Data() const;
		uint16 * uint16Data() const;
		int16 * int16Data() const;
		uint32 * uint32Data() const;
		int32 * int32Data() const;
		real32 * real32Data() const;
		real64 * real64Data() const;
		bool8 * boolData() const;
		char * stringData() const;

		unsigned long elements() const;
		const longVec& size() const;

		void seek(
			const longVec & from,
			const longVec & count );

		void dumpData( char ** values, unsigned long from, unsigned long count ) const;
		void dumpData( bool8 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int8 * values, unsigned long from, unsigned long count ) const;
		void dumpData( uint16 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int16 * values, unsigned long from, unsigned long count ) const;
		void dumpData( uint32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( real32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( real64 * values, unsigned long from, unsigned long count ) const;
		void loadData( char ** values, unsigned long from, unsigned long count ) const;
		void loadData( bool8 * values, unsigned long from, unsigned long count ) const;
		void loadData( int8 * values, unsigned long from, unsigned long count ) const;
		void loadData( uint16 * values, unsigned long from, unsigned long count ) const;
		void loadData( int16 * values, unsigned long from, unsigned long count ) const;
		void loadData( uint32 * values, unsigned long from, unsigned long count ) const;
		void loadData( int32 * values, unsigned long from, unsigned long count ) const;
		void loadData( real32 * values, unsigned long from, unsigned long count ) const;
		void loadData( real64 * values, unsigned long from, unsigned long count ) const;
		void copyData( unsigned long from, unsigned long to, unsigned long count ) const;
};

#endif
