using namespace std;

#include<cassert>
#include <ExtDal.h>
#include <sstream>
#include <Tracer.h>
#include <StopWatch.h>


class Fill : public SubTableProcessor
{

	public :

		Fill( Table * table ) : SubTableProcessor( table ) { PROC ENDPROC }

   		void proc( Table * table );
};


inline void
Fill::proc( Table * table )
{
	PROC

	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		Column * col = table -> column( i );
		ColumnData * coldat = col -> data();
		switch( col -> dataType() )
		{
			case Column::Bool :
			{
				bool8 * d = coldat -> boolData();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = ((col -> from() + j ) % 2 ? true : false );
				} // for
			}
				break;
			case Column::Int8 :
			{
				int8 * d = coldat -> int8Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = col -> from() + j;
				} // for
			}
				break;
			case Column::Int16 :
			{
				int16 * d = coldat -> int16Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = col -> from() + j;
				} // for
			}
				break;
			case Column::Int32 :
			{
				int32 * d = coldat -> int32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = col -> from() + j;
				} // for
			}
				break;
			case Column::Real32 :
			{
				real32 * d = coldat -> real32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = col -> from() + j;
				} // for
			}
				break;
			case Column::Real64 :
			{
				real64 * d = coldat -> real64Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					d[ j ] = col -> from() + j;
				} // for
			}
				break;
			case Column::DString :
			{
				char * d = coldat -> stringData();
				for(
					unsigned long j = 0;
					j < coldat -> count() * coldat -> parent() -> elements();
					++j )
				{
					d[ j ] = ('a' + ( coldat -> from() + j ) % 26 );
				} // for
			}
				break;
			default :
				break;
		} // switch
		delete coldat;
	} // for
	ENDPROC
}


class Check : public SubTableProcessor
{

	public :

		Check( Table * table ) : SubTableProcessor( table ) { PROC ENDPROC }


   		void proc( Table * table );
};


inline void
Check::proc( Table * table )
{
	PROC

	for( unsigned int i = 0; i < table -> columns(); ++i )
	{
		Column * col = table -> column( i );
		ColumnData * coldat = col -> data();
		switch( col -> dataType() )
		{
			case Column::Bool :
			{
				bool8 * booldat = coldat -> boolData();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( booldat[ j ] == ((col -> from()
					+ j ) % 2 ? true : false ));
				} // for
			}
				break;
			case Column::Int8 :
			{
				int8 * d = coldat -> int8Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int8(coldat -> from()
					+ j) );
				} // for
			}
				break;
			case Column::Int16 :
			{
				int16 * d = coldat -> int16Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int16(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Int32 :
			{
				int32 * d = coldat -> int32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == int32(coldat ->
					from() + j) );
				} // for
			}
				break;
			case Column::Real32 :
			{
				real32 * d = coldat -> real32Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::Real64 :
			{
				real64 * d = coldat -> real64Data();
				for( unsigned long j = 0; j < coldat -> count(); ++j )
				{
					assert( d[ j ] == coldat -> from() + j );
				} // for
			}
				break;
			case Column::DString :
			{
				char * d = coldat -> stringData();
				for(
					unsigned long j = 0;
					j < coldat -> count() * coldat -> parent() -> elements();
					++j )
				{
					assert( d[ j ] == (char)('a' + ( coldat
					-> from() + j ) % 26 ));
				} // for
			}
				break;
			default :
				break;
		} // switch
		delete coldat;
	} // for
	ENDPROC
}


int
main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );

  	unsigned long nRows = 10;

  	DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  	Table* tab = set->addTable("some table",nRows);

  	tab->addColumn("bool",Column::Bool,"bool data");
  	tab->addColumn("int8",Column::Int8,"int8 data","mm");
  	tab->addColumn("int16",Column::Int16,"int16 data","cm");
  	tab->addColumn("int32",Column::Int32,"int32 data","dm");
  	tab->addColumn("real32",Column::Real32,"real32 data","m");
  	tab->addColumn("real64",Column::Real64,"real64 data","Dm");
	vector<unsigned long>	size;
	size.push_back( 500 );
  	tab->addColumn("string",Column::DString,"string data","", size);
  	{
  		Fill	fill ( tab );
		fill.forEachSubTable();
		Check	check( tab );
		check.forEachSubTable();
	}

  	dataSetServer->close(set);


	set = dataSetServer->open("test.dat",DataSet::Read);
	tab = set->table("some table");
	{
		Check	check( tab );
		check.forEachSubTable();
	}
  	dataSetServer->close(set);

	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

  	return 0;
}

